#include "debug.h"
#include "MLX90640_API.h"
#include "MLX90640_I2C_Driver.h"

#define FPS2HZ   0x02
#define FPS4HZ   0x03
#define FPS8HZ   0x04
#define FPS16HZ  0x05
#define FPS32HZ  0x06

#define MLX90640_ADDR 0x33
#define RefreshRate FPS16HZ
#define TA_SHIFT 8 // Default shift for MLX90640 in open air

static uint16_t eeMLX90640[832];
static float mlx90640To[768];
uint16_t frame[834];
float emissivity = 0.95;
int status;

int main(void) {
    NVIC_PriorityGroupConfig(NVIC_PriorityGroup_2);
    SystemCoreClockUpdate();
    Delay_Init();
    USART_Printf_Init(115200);
    printf("Start...... \n");

    MLX90640_I2CInit();
    printf("I2C Initialized\n");

    MLX90640_SetRefreshRate(MLX90640_ADDR, RefreshRate);
    printf("Refresh rate set\n");
    MLX90640_SetChessMode(MLX90640_ADDR);
    printf("Chess mode set\n");

    paramsMLX90640 mlx90640;
    status = MLX90640_DumpEE(MLX90640_ADDR, eeMLX90640);
    if (status != 0) {
        printf("\r\nLoad system parameters error with code: %d\r\n", status);
    }
    printf("System parameters loaded\n");

    status = MLX90640_ExtractParameters(eeMLX90640, &mlx90640);
    if (status != 0) {
        printf("\r\nParameter extraction failed with error code: %d\r\n", status);
    }
    printf("Parameters extracted\n");

    while (1) {
        status = MLX90640_GetFrameData(MLX90640_ADDR, frame);
        if (status < 0) {
            printf("\r\nGet frame data error with code: %d\r\n", status);
            continue;
        }
        printf("Frame data received\n");

        float vdd = MLX90640_GetVdd(frame, &mlx90640);
        float ta = MLX90640_GetTa(frame, &mlx90640);

        float tr = ta - TA_SHIFT; // Reflected temperature compensation
        MLX90640_CalculateTo(frame, &mlx90640, emissivity, tr, mlx90640To);

        printf("Vdd: %0.2f V, Ta: %0.2f C\n", vdd, ta);
        for (int i = 0; i < 768; i++) {
            if (i % 32 == 0) {
                printf("\n");
            }
            printf("%0.2f ", mlx90640To[i]);
        }
        printf("\n");

        Delay_Ms(1000); // ӳ1룬Ҫ
    }
}
